#include "Config.h"
#define CONFIG_SIZE 0x4C
Config::Config()
{
    if (!LittleFS.begin())
    {
        Serial.println("An Error has occurred while mounting LittleFS");
        return;
    }
}
Config::~Config()
{
    free(buffer);
}

bool Config::loadBin()
{
    File config_file = LittleFS.open("/settings", "r");
    if(buffer==nullptr)
    buffer = (uint8_t*) malloc(CONFIG_SIZE);
    if (config_file.available())
    {
        config_file.read(buffer, CONFIG_SIZE);
        SSID = (char*) buffer;
        buffer[0x1F] = 0x00;
        buffer[0x3F] = 0x00;
        PASS = (char*)(buffer + 0x20);
        ip = (uint32_t *)(buffer + 0x40);
        subnet = (uint32_t *)(buffer + 0x44);
        gw = (uint32_t *)(buffer + 0x48);
        mode = (uint8_t *)(buffer + 0x4C);
    }
    config_file.close();
    return true;
}

bool Config::saveBin()
{
    File config_file = LittleFS.open("/settings", "w");
    config_file.write(buffer,CONFIG_SIZE);
    delay(100);
    config_file.close();
    return true;
}
bool Config::setSSID(const char *ssid){
    size_t offset = 0;
    bool copy = true;
    for(int i=0; i<31;i++){
        if(ssid[i]==0x00)
            copy = false;
        buffer[offset+i] = copy?ssid[i]:0x00;
    }
    buffer[offset+31] = '\n';
    return true;
}
bool Config::setPASS(const char *pass){
    size_t offset = 0x20;
    bool copy = true;
    for(int i=0; i<31;i++){
        if(pass[i]==0x00)
            copy = false;
        buffer[offset+i] = copy?pass[i]:0x00;
    }
    buffer[offset+31] = '\n';
    return true;
}
void Config::print(){
    Serial.print("BufferHEX ");
    for(int i=0; i<CONFIG_SIZE;i++){
       Serial.print(i,HEX); 
       Serial.print(">");
       Serial.print(buffer[i],HEX);
       Serial.print("|");
    }
    Serial.print("SSID: ");
    Serial.println(SSID);
    Serial.print("PASS: ");
    Serial.println(PASS);    
    Serial.print("IP");
    Serial.print((uint8_t) *ip, DEC);
    Serial.print(".");
    Serial.print((uint8_t)*((uint8_t*)ip+1), DEC);
    Serial.print(".");
    Serial.print((uint8_t) *((uint8_t*)ip+2), DEC);
    Serial.print(".");
    Serial.println((uint8_t) *((uint8_t*)ip+3), DEC);
    Serial.print("GW");
    Serial.print((uint8_t) *((uint8_t*)gw), DEC);
    Serial.print(".");
    Serial.print((uint8_t) *((uint8_t*)gw+1), DEC);
    Serial.print(".");
    Serial.print((uint8_t) *((uint8_t*)gw+2), DEC);
    Serial.print(".");
    Serial.println((uint8_t) *((uint8_t*)gw+3), DEC);
    Serial.print("SUBNET");
    Serial.print((uint8_t)*((uint8_t*)subnet), DEC);
    Serial.print(".");
    Serial.print((uint8_t)*((uint8_t*)subnet+1), DEC);
    Serial.print(".");
    Serial.print((uint8_t)*((uint8_t*)subnet+2), DEC);
    Serial.print(".");
    Serial.println((uint8_t)*((uint8_t*)subnet+3), DEC);


    
}