//#define DEBUG
#include <Arduino.h>
#include <Wire.h>
#include "Keyboard.h"
#include "Rfid.h"
#include "Interface.h"
#include <ESP8266WiFi.h>
#include "WebConsole.h"
#include "UserDb.h"
#include "Config.h"
#include <ESP8266mDNS.h>
#include <DNSServer.h>
// File config
Config config;
userdb::UserDb userdatabase("userdb.csv");
webconsole::WebConsole web;
// Rfid
Rfid rfid;
// i2C Bus
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4

Keyboard keyboard(200);

Interface iface;
// Wifi control
IPAddress local_IP(192, 168, 4, 22);
IPAddress gateway(0, 0, 0, 0);
IPAddress subnet(255, 255, 255, 0);
IPAddress dns(192, 168, 178, 1);
DNSServer dnsServer;
void setup()
{
	//config.loadConfig();
	config.loadBin();
	Serial.begin(115200);
	Serial.println("Starting System");
	Serial.print("\t1. Network config ->");
	WiFi.mode(WIFI_AP);
	Serial.println(WiFi.softAPConfig(*config.ip, *config.gw, *config.subnet) ? "Ready" : "Failed!");
	Serial.print("\t2 AP setup " + String(config.SSID) + " -> ");
	if (strlen(config.PASS) > 0)
		Serial.println(WiFi.softAP(config.SSID, config.PASS) ? "Ready" : "Failed!");
	else
		Serial.println(WiFi.softAP(config.SSID) ? "Ready" : "Failed!");
	WiFi.hostname("Doorlock");
	dnsServer.start(53, "*", local_IP); // DNS spoofing (Only HTTP)
	delay(150);
#ifdef DEBUG
	userdatabase.print_to_serial();
#endif
	web.init(&userdatabase);
	keyboard.begin(&Wire);
	rfid.begin();
	web.attachRfid(&rfid);
	iface.begin(&keyboard);
}

void loop()
{
	dnsServer.processNextRequest();
	rfid.scan();
	web.serve();
	keyboard.scanAsync();
	userdb::User login_user;
	if (web.isInterceptingRfid() ==3)
		iface.showMessage("WebUI connected", "Awaiting RFID", 2000);
	else if (web.isInterceptingRfid() == 1)
		iface.showMessage("WebUI connected", "RFID Found", 1000);
	else if (iface.pinAvailable() && iface.getState() != Interface::GREET)
	{
		unsigned long delta = millis();
		login_user = userdatabase.user_by_pin(iface.getPin());
		Serial.println("Query duration: " + String(millis() - delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Pin incorrect", 3000);
			return;
		}
	}
	else if (rfid.available() && iface.getState() != Interface::GREET)
	{
		unsigned long delta = millis();
		login_user = userdatabase.user_by_rfid(rfid.getID());
		Serial.println("Query duration: " + String(millis() - delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Unkown Card", 3000);
			return;
		}
	}

	if (login_user.enabled == true)
	{
		iface.greetUser(login_user.first_name + " " + login_user.last_name);
		Serial.println("Logon from User " + login_user.toString());
	}
	iface.render();
}